<?php declare(strict_types=1);

namespace Infomaker\ProjectPlugin;

use Infomaker\Everyware\Support\Environment;
use Infomaker\Everyware\Support\Storage\ObjectDB;
use Infomaker\Everyware\Support\Str;
use Infomaker\ProjectPlugin\Admin\Abstracts\ComponentsPage;
use Infomaker\ProjectPlugin\Admin\ProjectPluginPage\ComponentsHandler;
use Infomaker\ProjectPlugin\Admin\ProjectPluginPage\PluginsPage;
use Infomaker\ProjectPlugin\Admin\ProjectPluginPage\SettingsPage;
use Infomaker\ProjectPlugin\Admin\ProjectPluginPage\WidgetsPage;
use Infomaker\ProjectPlugin\Admin\SettingsPage\SettingsParametersPage;
use Infomaker\ProjectPlugin\Interfaces\ComponentsPageInterface;
use Infomaker\ProjectPlugin\Interfaces\PluginConfigInterface;
use Infomaker\ProjectPlugin\Interfaces\PluginPageInterface;
use Infomaker\ProjectPlugin\Interfaces\SettingsPageInterface;

class ProjectPlugin
{
    protected static $instance;
    protected static $version = '0.1';

    /** @var SettingsPage */
    protected $settingsPage;

    protected $componentsPages;

    /** @var PluginConfigInterface */
    protected $config;

    protected $pages = [];

    private $textDomain = 'everyware_project_plugin_text_domain';

    private $actionLinks = [];

    public static function bootstrap(PluginConfigInterface $config): void
    {
        /** @var self $plugin */
        $plugin = static::getInstance();

        $plugin->init($config);

        /** @noinspection PhpUndefinedFunctionInspection */
        if(is_admin()) {
            /** @noinspection PhpUndefinedFunctionInspection */
            add_action('plugins_loaded', [$plugin, 'getInstance']);
        }
    }

    public static function getInstance(): self
    {
        if(static::$instance === null) {
            static::$instance = new static;
        }

        return static::$instance;
    }

    public function init(PluginConfigInterface $config): void
    {
        $this->config = $config;

        $this->setupScripts()
             ->setupActionLinks()
             ->setupAdminMenu()
             ->registerHooks();
    }

    private function registerHooks(): self
    {
        $bootstrapFilePath = $this->config->getPath('bootstrap.php');

        // Hooks for activation and deactivation of plugin
        register_activation_hook($bootstrapFilePath, function() {
            $this->activate();
        });
        register_deactivation_hook($bootstrapFilePath, function() {
            $this->deactivate();
        });

        return $this;
    }

    private function setupActionLinks(): self
    {
        $bootstrapFilePath = $this->config->getPath('bootstrap.php');
        $pluginSlug        = $this->config->getSlug();

        /** @noinspection PhpUndefinedFunctionInspection */
        $this->addActionLink(__('Settings', $this->textDomain), admin_url("admin.php?page={$pluginSlug}"));

        // Add settings link for this plugin in WordPress plugin
        add_filter("plugin_action_links_{$bootstrapFilePath}", function($links) {
            return $this->applyActionLinks($links);
        });

        return $this;
    }

    protected function setupScripts(): self
    {
        add_action('admin_init', function() {
            wp_enqueue_media();
        });

        add_action('admin_enqueue_scripts', function() {
            if(is_admin()) {
                $assetsPath    = plugin_dir_url(__FILE__) . '/assets';
                $scriptVersion = Environment::isDev() ? false : GIT_COMMIT;

                wp_enqueue_style('everyware-project-plugin-style', "{$assetsPath}/css/admin.css", [], $scriptVersion, 'all');
                wp_enqueue_script('everyware-project-plugin-js', "{$assetsPath}/js/admin.js", ['jquery'], $scriptVersion, true);
            }
        });

        return $this;
    }

    public function addPage(ComponentsPageInterface $page): self
    {
        if($page instanceof ComponentsPageInterface) {
            $this->addComponentsPage($page);
        }

        $this->pages[] = $page;

        return $this;
    }

    protected function addComponentsPage(ComponentsPageInterface $page): self
    {
        $this->componentsPages[] = $page;

        return $this;
    }

    public function addSettingsTab(SettingsPageInterface $page): self
    {
        $this->settingsPage->addSettingsTab($page);

        return $this;
    }


    public function addActionLink(string $linkTitle, string $linkUrl, string $linkSlug = null): self
    {
        $linkSlug = $linkSlug ?? Str::slug($linkTitle);

        $this->actionLinks[$linkSlug] = "<a href=\"{$linkUrl}\">{$linkTitle}</a>";

        return $this;
    }

    protected function setupAdminMenu(): self
    {
        $this->settingsPage = new SettingsPage();
        $this->addSettingsTab(new SettingsParametersPage());

        $this->addPage(new PluginsPage(new ComponentsHandler('../../app/Plugins', 'everyware_project_plugin_active_plugins')))
             ->addPage(new WidgetsPage(new ComponentsHandler('../../app/Widgets', 'everyware_project_plugin_active_widgets')));

        // Add menu items
        add_action('admin_menu', function() {

            $menuSlug = $this->config->getSlug();

            // Add head menu item
            add_menu_page($this->config->getName(), $this->config->getName(), 'manage_options', $menuSlug);

            // Add Settings menu item
            $this->settingsPage->addToMenu($menuSlug);

            foreach($this->pages as $page) {
                $page->addToMenu($menuSlug);
            }

        }, 10);

        return $this;
    }

    protected function applyActionLinks(array $links): array
    {
        return array_merge($this->actionLinks, $links);
    }

    protected function activate(): void
    {
        /** @var ComponentsPage $componentsPage */
        foreach($this->componentsPages as $componentsPage) {
            $componentsPage->activateComponents();
        }
    }

    protected function deactivate(): void
    {
        $filteredPlugins = static::getAllPlugins();

        /** @var ComponentsPage $componentsPage */
        foreach($this->componentsPages as $componentsPage) {
            $componentsPage->deactivateComponents();
            $filteredPlugins = $componentsPage->getFilteredPlugins($filteredPlugins);
        }

        ObjectDB::update('active_plugins', $filteredPlugins);
    }

    public static function getAllPlugins()
    {
        return ObjectDB::get('active_plugins', []);
    }

    public function getPageUrl(ComponentsPageInterface $page): string
    {
        return admin_url('admin.php?page=' . $page->getSlug());
    }
}
